package com.caimei365.cloud.config;

import com.caimei365.cloud.service.TouristService;
import org.apache.commons.lang.StringUtils;
import org.apache.rocketmq.spring.core.RocketMQTemplate;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Configuration;
import org.springframework.http.server.reactive.ServerHttpRequest;
import org.springframework.stereotype.Component;
import org.springframework.web.reactive.config.WebFluxConfigurer;
import org.springframework.web.server.ServerWebExchange;
import org.springframework.web.server.WebFilter;
import org.springframework.web.server.WebFilterChain;
import reactor.core.publisher.Mono;

import javax.annotation.PostConstruct;
import javax.annotation.Resource;
import java.net.InetSocketAddress;
import java.util.Objects;

/**
 * 全局过滤器
 *
 * @author : Charles
 * @date : 2021/3/24
 */
@Configuration
public class WebConfiguration implements WebFluxConfigurer {

     @Autowired
     private TouristService touristService;

     private static WebConfiguration webConfiguration;

     @PostConstruct
     public void init() {
         webConfiguration = this;
         webConfiguration.touristService = this.touristService;
     }
    /**
     * 获取客户端IP
     * https://stackoverflow.com/questions/51192630/how-do-you-get-clients-ip-address-spring-webflux-websocket?rq=1
     * https://stackoverflow.com/questions/50981136/how-to-get-client-ip-in-webflux
     * https://docs.spring.io/spring/docs/current/spring-framework-reference/web-reactive.html#webflux-filters
     * 由于在低版本的 spring-webflux 中不支持直接获得请求 IP（https://jira.spring.io/browse/SPR-16681），
     * 因此从org.springframework.web.server.ServerWebExchange中获得 IP 后，在放到 header 里
     */
    @Component
    public static class RetrieveClientIpWebFilter implements WebFilter {
        @Override
        public Mono<Void> filter(ServerWebExchange exchange, WebFilterChain chain) {
            ServerHttpRequest request = exchange.getRequest();
            // 从我们的 nginx 重写请求获取的参数，X-Real-IP是在nginx中配置的，要对应上；
            String clientIp = request.getHeaders().getFirst("X-Real-IP");
            if (StringUtils.isEmpty(clientIp)) {
                InetSocketAddress remoteAddress = request.getRemoteAddress();
                clientIp = Objects.requireNonNull(remoteAddress).getAddress().getHostAddress();
            }
            ServerHttpRequest mutatedServerHttpRequest = request.mutate().header("X-CLIENT-IP", clientIp).build();
            ServerWebExchange mutatedServerWebExchange = exchange.mutate().request(mutatedServerHttpRequest).build();
            chain.filter(mutatedServerWebExchange);
            // 游客编号
            //TouristService touristService = new TouristService();
            String touristId = webConfiguration.touristService.touristInfo();
            return exchange.getSession().flatMap(session -> {
                session.getAttributes().put("touristId", touristId);
                return chain.filter(exchange);
            });
        }
    }

}
